@@ -1,12 +1,14 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: March 15, 2016
+# Last Change: September 29, 2016
 # URL: https://humanfriendly.readthedocs.org
 
 """The main module of the `humanfriendly` package."""
 
 # Standard library modules.
+import collections
+import decimal
 import multiprocessing
 import numbers
 import os
@@ -37,7 +39,7 @@ from humanfriendly.prompts import prompt_for_choice  # NOQA
 from humanfriendly.compat import is_string
 
 # Semi-standard module versioning.
-__version__ = '1.44.4'
+__version__ = '2.0.1'
 
 # Spinners are redrawn at most this many seconds.
 minimum_spinner_interval = 0.2
@@ -50,13 +52,17 @@ erase_line_code = '\r\x1b[K'
 hide_cursor_code = '\x1b[?25l'
 show_cursor_code = '\x1b[?25h'
 
-# Common disk size units, used for formatting and parsing.
-disk_size_units = (dict(prefix='b', divider=1, singular='byte', plural='bytes'),
-                   dict(prefix='k', divider=1024**1, singular='KB', plural='KB'),
-                   dict(prefix='m', divider=1024**2, singular='MB', plural='MB'),
-                   dict(prefix='g', divider=1024**3, singular='GB', plural='GB'),
-                   dict(prefix='t', divider=1024**4, singular='TB', plural='TB'),
-                   dict(prefix='p', divider=1024**5, singular='PB', plural='PB'))
+SizeUnit = collections.namedtuple('SizeUnit', 'divider, symbol, name')
+CombinedUnit = collections.namedtuple('CombinedUnit', 'decimal, binary')
+
+# Common disk size units in binary (base-2) and decimal (base-10) multiples.
+disk_size_units = (
+    CombinedUnit(SizeUnit(1000**1, 'KB', 'kilobyte'), SizeUnit(1024**1, 'KiB', 'kibibyte')),
+    CombinedUnit(SizeUnit(1000**2, 'MB', 'megabyte'), SizeUnit(1024**2, 'MiB', 'mebibyte')),
+    CombinedUnit(SizeUnit(1000**3, 'GB', 'gigabyte'), SizeUnit(1024**3, 'GiB', 'gibibyte')),
+    CombinedUnit(SizeUnit(1000**4, 'TB', 'terabyte'), SizeUnit(1024**4, 'TiB', 'tebibyte')),
+    CombinedUnit(SizeUnit(1000**5, 'PB', 'petabyte'), SizeUnit(1024**5, 'PiB', 'pebibyte')),
+)
 
 # Common length size units, used for formatting and parsing.
 length_size_units = (dict(prefix='nm', divider=1e-09, singular='nm', plural='nm'),
@@ -69,10 +75,10 @@ length_size_units = (dict(prefix='nm', divider=1e-09, singular='nm', plural='nm'
 time_units = (dict(divider=1e-3, singular='millisecond', plural='milliseconds', abbreviation='ms'),
               dict(divider=1, singular='second', plural='seconds', abbreviation='s'),
               dict(divider=60, singular='minute', plural='minutes', abbreviation='m'),
-              dict(divider=60*60, singular='hour', plural='hours', abbreviation='h'),
-              dict(divider=60*60*24, singular='day', plural='days', abbreviation='d'),
-              dict(divider=60*60*24*7, singular='week', plural='weeks', abbreviation='w'),
-              dict(divider=60*60*24*7*52, singular='year', plural='years', abbreviation='y'))
+              dict(divider=60 * 60, singular='hour', plural='hours', abbreviation='h'),
+              dict(divider=60 * 60 * 24, singular='day', plural='days', abbreviation='d'),
+              dict(divider=60 * 60 * 24 * 7, singular='week', plural='weeks', abbreviation='w'),
+              dict(divider=60 * 60 * 24 * 7 * 52, singular='year', plural='years', abbreviation='y'))
 
 
 def coerce_boolean(value):
@@ -103,18 +109,19 @@ def coerce_boolean(value):
         return bool(value)
 
 
-def format_size(num_bytes, keep_width=False):
+def format_size(num_bytes, keep_width=False, binary=False):
     """
     Format a byte count as a human readable file size.
 
     :param num_bytes: The size to format in bytes (an integer).
-    :param keep_width: ``True`` if trailing zeros should not be stripped,
-                       ``False`` if they can be stripped.
+    :param keep_width: :data:`True` if trailing zeros should not be stripped,
+                       :data:`False` if they can be stripped.
+    :param binary: :data:`True` to use binary multiples of bytes (base-2),
+                   :data:`False` to use decimal multiples of bytes (base-10).
     :returns: The corresponding human readable file size (a string).
 
-    This function supports ranges from kilobytes to terabytes.
-
-    Some examples:
+    This function knows how to format sizes in bytes, kilobytes, megabytes,
+    gigabytes, terabytes and petabytes. Some examples:
 
     >>> from humanfriendly import format_size
     >>> format_size(0)
@@ -123,50 +130,79 @@ def format_size(num_bytes, keep_width=False):
     '1 byte'
     >>> format_size(5)
     '5 bytes'
-    >>> format_size(1024 ** 2)
-    '1 MB'
-    >>> format_size(1024 ** 3 * 4)
+    > format_size(1000)
+    '1 KB'
+    > format_size(1024, binary=True)
+    '1 KiB'
+    >>> format_size(1000 ** 3 * 4)
     '4 GB'
     """
     for unit in reversed(disk_size_units):
-        if num_bytes >= unit['divider']:
-            number = round_number(float(num_bytes) / unit['divider'], keep_width=keep_width)
-            return pluralize(number, unit['singular'], unit['plural'])
+        if num_bytes >= unit.binary.divider and binary:
+            number = round_number(float(num_bytes) / unit.binary.divider, keep_width=keep_width)
+            return pluralize(number, unit.binary.symbol, unit.binary.symbol)
+        elif num_bytes >= unit.decimal.divider and not binary:
+            number = round_number(float(num_bytes) / unit.decimal.divider, keep_width=keep_width)
+            return pluralize(number, unit.decimal.symbol, unit.decimal.symbol)
     return pluralize(num_bytes, 'byte')
 
 
-def parse_size(size):
+def parse_size(size, binary=False):
     """
     Parse a human readable data size and return the number of bytes.
 
     :param size: The human readable file size to parse (a string).
+    :param binary: :data:`True` to use binary multiples of bytes (base-2) for
+                   ambiguous unit symbols and names, :data:`False` to use
+                   decimal multiples of bytes (base-10).
     :returns: The corresponding size in bytes (an integer).
     :raises: :exc:`InvalidSize` when the input can't be parsed.
 
-    Some examples:
+    This function knows how to parse sizes in bytes, kilobytes, megabytes,
+    gigabytes, terabytes and petabytes. Some examples:
 
     >>> from humanfriendly import parse_size
     >>> parse_size('42')
     42
+    >>> parse_size('13b')
+    13
+    >>> parse_size('5 bytes')
+    5
     >>> parse_size('1 KB')
+    1000
+    >>> parse_size('1 kilobyte')
+    1000
+    >>> parse_size('1 KiB')
+    1024
+    >>> parse_size('1 KB', binary=True)
     1024
-    >>> parse_size('5 kilobyte')
-    5120
     >>> parse_size('1.5 GB')
+    1500000000
+    >>> parse_size('1.5 GB', binary=True)
     1610612736
     """
     tokens = tokenize(size)
     if tokens and isinstance(tokens[0], numbers.Number):
-        # If the input contains only a number, it's assumed to be the number of bytes.
-        if len(tokens) == 1:
+        # Get the normalized unit (if any) from the tokenized input.
+        normalized_unit = tokens[1].lower() if len(tokens) == 2 and is_string(tokens[1]) else ''
+        # If the input contains only a number, it's assumed to be the number of
+        # bytes. The second token can also explicitly reference the unit bytes.
+        if len(tokens) == 1 or normalized_unit.startswith('b'):
             return int(tokens[0])
-        # Otherwise we expect to find two tokens: A number and a unit.
-        if len(tokens) == 2 and is_string(tokens[1]):
-            normalized_unit = tokens[1].lower()
-            # Try to match the first letter of the unit.
+        # Otherwise we expect two tokens: A number and a unit.
+        if normalized_unit:
             for unit in disk_size_units:
-                if normalized_unit.startswith(unit['prefix']):
-                    return int(tokens[0] * unit['divider'])
+                # First we check for unambiguous symbols (KiB, MiB, GiB, etc)
+                # and names (kibibyte, mebibyte, gibibyte, etc) because their
+                # handling is always the same.
+                if normalized_unit in (unit.binary.symbol.lower(), unit.binary.name.lower()):
+                    return int(tokens[0] * unit.binary.divider)
+                # Now we will deal with ambiguous prefixes (K, M, G, etc),
+                # symbols (KB, MB, GB, etc) and names (kilobyte, megabyte,
+                # gigabyte, etc) according to the caller's preference.
+                if (normalized_unit in (unit.decimal.symbol.lower(), unit.decimal.name.lower()) or
+                        normalized_unit.startswith(unit.decimal.symbol[0].lower())):
+                    return int(tokens[0] * (unit.binary.divider if binary else unit.decimal.divider))
     # We failed to parse the size specification.
     msg = "Failed to parse size! (input %r was tokenized as %r)"
     raise InvalidSize(msg % (size, tokens))
@@ -309,7 +345,7 @@ def round_number(count, keep_width=False):
     return text
 
 
-def format_timespan(num_seconds, detailed=False):
+def format_timespan(num_seconds, detailed=False, max_units=3):
     """
     Format a timespan in seconds as a human readable string.
 
@@ -317,6 +353,8 @@ def format_timespan(num_seconds, detailed=False):
     :param detailed: If :data:`True` milliseconds are represented separately
                      instead of being represented as fractional seconds
                      (defaults to :data:`False`).
+    :param max_units: The maximum number of units to show in the formatted time
+                      span (an integer, defaults to three).
     :returns: The formatted timespan as a string.
 
     Some examples:
@@ -341,18 +379,30 @@ def format_timespan(num_seconds, detailed=False):
     else:
         # Slow path.
         result = []
-        for unit in reversed(time_units):
-            if num_seconds >= unit['divider']:
-                count = int(num_seconds / unit['divider'])
-                num_seconds %= unit['divider']
+        num_seconds = decimal.Decimal(str(num_seconds))
+        relevant_units = list(reversed(time_units[0 if detailed else 1:]))
+        for unit in relevant_units:
+            # Extract the unit count from the remaining time.
+            divider = decimal.Decimal(str(unit['divider']))
+            count = num_seconds / divider
+            num_seconds %= divider
+            # Round the unit count appropriately.
+            if unit != relevant_units[-1]:
+                # Integer rounding for all but the smallest unit.
+                count = int(count)
+            else:
+                # Floating point rounding for the smallest unit.
+                count = round_number(count)
+            # Only include relevant units in the result.
+            if count not in (0, '0'):
                 result.append(pluralize(count, unit['singular'], unit['plural']))
         if len(result) == 1:
             # A single count/unit combination.
             return result[0]
         else:
             if not detailed:
-                # Remove insignificant data from the formatted timespan.
-                result = result[:3]
+                # Remove `insignificant' data from the formatted timespan.
+                result = result[:max_units]
             # Format the timespan in a readable way.
             return concatenate(result)
 
@@ -737,7 +787,7 @@ class Spinner(object):
                 if not label:
                     raise Exception("No label set for spinner!")
                 elif self.total and progress:
-                    label = "%s: %.2f%%" % (label, progress/(self.total/100.0))
+                    label = "%s: %.2f%%" % (label, progress / (self.total / 100.0))
                 elif self.timer and self.timer.elapsed_time > 2:
                     label = "%s (%s)" % (label, self.timer.rounded)
                 self.stream.write("%s %s %s ..\r" % (erase_line_code, state, label))
