@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: October 22, 2015
+# Last Change: September 28, 2016
 # URL: https://humanfriendly.readthedocs.org
 
 """
@@ -36,7 +36,8 @@ except ImportError:
 # Modules included in our package. We import find_meta_variables() here to
 # preserve backwards compatibility with older versions of humanfriendly where
 # that function was defined in this module.
-from humanfriendly.text import format
+from humanfriendly.compat import is_unicode
+from humanfriendly.text import concatenate, format
 from humanfriendly.usage import find_meta_variables, format_usage  # NOQA
 
 ANSI_CSI = '\x1b['
@@ -59,6 +60,13 @@ A dictionary with (name, number) pairs of `portable color codes`_. Used by
 .. _portable color codes: http://en.wikipedia.org/wiki/ANSI_escape_code#Colors
 """
 
+ANSI_TEXT_STYLES = dict(bold=1, faint=2, underline=4, inverse=7, strike_through=9)
+"""
+A dictionary with (name, number) pairs of text styles (effects). Used by
+:func:`ansi_style()` to generate ANSI escape sequences that change text
+styles. Only widely supported text styles are included here.
+"""
+
 DEFAULT_LINES = 25
 """The default number of lines in a terminal (an integer)."""
 
@@ -78,11 +86,11 @@ def message(*args, **kw):
     """
     Show an informational message on the terminal.
 
-    :param args: Any position arguments are passed on to :func:`format()`.
-    :param kw: Any keyword arguments are passed on to :func:`format()`.
+    :param args: Any position arguments are passed on to :func:`~humanfriendly.text.format()`.
+    :param kw: Any keyword arguments are passed on to :func:`~humanfriendly.text.format()`.
 
-    Renders the message using :func:`format()` and writes the resulting
-    string to :data:`sys.stderr` (followed by a newline).
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string to :data:`sys.stderr` (followed by a newline).
     """
     sys.stderr.write(format(*args, **kw) + '\n')
 
@@ -91,16 +99,17 @@ def warning(*args, **kw):
     """
     Show a warning message on the terminal.
 
-    :param args: Any position arguments are passed on to :func:`format()`.
-    :param kw: Any keyword arguments are passed on to :func:`format()`.
+    :param args: Any position arguments are passed on to :func:`~humanfriendly.text.format()`.
+    :param kw: Any keyword arguments are passed on to :func:`~humanfriendly.text.format()`.
 
-    Renders the message using :func:`format()` and writes the resulting string
-    to :data:`sys.stderr` (followed by a newline). If :data:`sys.stderr` is
-    connected to a terminal :func:`ansi_wrap()` is used to color the message in
-    a red font (to make the warning stand out from surrounding text).
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string to :data:`sys.stderr` (followed by a newline). If
+    :data:`sys.stderr` is connected to a terminal :func:`ansi_wrap()` is used
+    to color the message in a red font (to make the warning stand out from
+    surrounding text).
     """
     text = format(*args, **kw)
-    if connected_to_terminal(sys.stderr):
+    if terminal_supports_colors(sys.stderr):
         text = ansi_wrap(text, color='red')
     sys.stderr.write(text + '\n')
 
@@ -122,8 +131,7 @@ def ansi_strip(text, readline_hints=True):
     return text
 
 
-def ansi_style(color=None, bold=False, faint=False, underline=False,
-               inverse=False, strike_through=False, readline_hints=False):
+def ansi_style(**kw):
     """
     Generate ANSI escape sequences for the given color and/or style(s).
 
@@ -131,40 +139,30 @@ def ansi_style(color=None, bold=False, faint=False, underline=False,
                   'green', 'yellow', 'blue', 'magenta', 'cyan' or 'white') or
                   :data:`None` (the default) which means no escape sequence to
                   switch color will be emitted.
-    :param bold: :data:`True` enables bold font (the default is :data:`False`).
-    :param faint: :data:`True` enables faint font (the default is
-                  :data:`False`).
-    :param underline: :data:`True` enables underline font (the default is
-                      :data:`False`).
-    :param inverse: :data:`True` enables inverse font (the default is
-                    :data:`False`).
-    :param strike_through: :data:`True` enables crossed-out / strike-through
-                           font (the default is :data:`False`).
     :param readline_hints: If :data:`True` then :func:`readline_wrap()` is
-                           applied to the generated ANSI escape sequences.
+                           applied to the generated ANSI escape sequences (the
+                           default is :data:`False`).
+    :param kw: Any additional keyword arguments are expected to match an entry
+               in the :data:`ANSI_TEXT_STYLES` dictionary. If the argument's
+               value evaluates to :data:`True` the respective style will be
+               enabled.
     :returns: The ANSI escape sequences to enable the requested text styles or
               an empty string if no styles were requested.
     :raises: :py:exc:`~exceptions.ValueError` when an invalid color name is given.
     """
-    sequences = []
-    if bold:
-        sequences.append('1')
-    if faint:
-        sequences.append('2')
-    if underline:
-        sequences.append('4')
-    if inverse:
-        sequences.append('7')
-    if strike_through:
-        sequences.append('9')
-    if color:
-        if color not in ANSI_COLOR_CODES:
+    # Start with sequences that change text styles.
+    sequences = [str(ANSI_TEXT_STYLES[k]) for k, v in kw.items() if k in ANSI_TEXT_STYLES and v]
+    # Append the color code (if any).
+    color_name = kw.get('color')
+    if color_name:
+        # Validate the color name.
+        if color_name not in ANSI_COLOR_CODES:
             msg = "Invalid color name %r! (expected one of %s)"
-            raise ValueError(msg % (color, ", ".join(sorted(ANSI_COLOR_CODES))))
-        sequences.append('3%i' % ANSI_COLOR_CODES[color])
+            raise ValueError(msg % (color_name, concatenate(sorted(ANSI_COLOR_CODES))))
+        sequences.append('3%i' % ANSI_COLOR_CODES[color_name])
     if sequences:
         encoded = ANSI_CSI + ';'.join(sequences) + ANSI_SGR
-        return readline_wrap(encoded) if readline_hints else encoded
+        return readline_wrap(encoded) if kw.get('readline_hints') else encoded
     else:
         return ''
 
@@ -235,22 +233,38 @@ def connected_to_terminal(stream=None):
     """
     Check if a stream is connected to a terminal.
 
-    If this function returns :data:`True` on a UNIX system it generally means
-    that ANSI escape sequences are supported and can be used.
-
-    :param stream: The stream to check (a :class:`file` object, defaults to
-                   :data:`sys.stdout`).
+    :param stream: The stream to check (a file-like object,
+                   defaults to :data:`sys.stdout`).
     :returns: :data:`True` if the stream is connected to a terminal,
               :data:`False` otherwise.
+
+    See also :func:`terminal_supports_colors()`.
     """
-    if stream is None:
-        stream = sys.stdout
+    stream = sys.stdout if stream is None else stream
     try:
         return stream.isatty()
     except Exception:
         return False
 
 
+def terminal_supports_colors(stream=None):
+    """
+    Check if a stream is connected to a terminal that supports ANSI escape sequences.
+
+    :param stream: The stream to check (a file-like object,
+                   defaults to :data:`sys.stdout`).
+    :returns: :data:`True` if the terminal supports ANSI escape sequences,
+              :data:`False` otherwise.
+
+    This function is inspired by the implementation of
+    `django.core.management.color.supports_color()
+    <https://github.com/django/django/blob/master/django/core/management/color.py>`_.
+    """
+    return (sys.platform != 'Pocket PC' and
+            (sys.platform != 'win32' or 'ANSICON' in os.environ) and
+            connected_to_terminal(stream))
+
+
 def find_terminal_size():
     """
     Determine the number of lines and columns visible in the terminal.
@@ -349,16 +363,18 @@ def usage(usage_text):
        using :func:`.format_usage()`.
     2. The usage message is shown using a pager (see :func:`show_pager()`).
     """
-    if connected_to_terminal(sys.stdout):
+    if terminal_supports_colors(sys.stdout):
         usage_text = format_usage(usage_text)
     show_pager(usage_text)
 
 
-def show_pager(formatted_text):
+def show_pager(formatted_text, encoding='UTF-8'):
     """
     Print a large text to the terminal using a pager.
 
     :param formatted_text: The text to print to the terminal (a string).
+    :param encoding: The name of the text encoding used to encode the formatted
+                     text if the formatted text is a Unicode string (a string).
 
     The use of a pager helps to avoid the wall of text effect where the user
     has to scroll up to see where the output began (not very user friendly).
@@ -376,6 +392,8 @@ def show_pager(formatted_text):
             pager_command = ['less', '--RAW-CONTROL-CHARS']
         else:
             pager_command = [os.environ.get('PAGER', 'less')]
+        if is_unicode(formatted_text):
+            formatted_text = formatted_text.encode(encoding)
         pager = subprocess.Popen(pager_command, stdin=subprocess.PIPE)
         pager.communicate(input=formatted_text)
     else:
