@@ -4,7 +4,7 @@
 # Tests for the `humanfriendly' package.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: February 18, 2016
+# Last Change: September 29, 2016
 # URL: https://humanfriendly.readthedocs.org
 
 """Test suite for the `humanfriendly` package."""
@@ -14,6 +14,7 @@ import logging
 import math
 import os
 import random
+import string
 import sys
 import time
 import unittest
@@ -49,11 +50,13 @@ from humanfriendly.terminal import (
     ansi_wrap,
     connected_to_terminal,
     find_terminal_size,
+    show_pager,
     terminal_supports_colors,
 )
 from humanfriendly.usage import (
     find_meta_variables,
     format_usage,
+    parse_usage,
     render_usage,
 )
 
@@ -118,30 +121,38 @@ class HumanFriendlyTestCase(unittest.TestCase):
         day = hour * 24
         week = day * 7
         year = week * 52
-        self.assertEqual('1 millisecond', humanfriendly.format_timespan(0.001, detailed=True))
-        self.assertEqual('500 milliseconds', humanfriendly.format_timespan(0.5, detailed=True))
-        self.assertEqual('0.5 seconds', humanfriendly.format_timespan(0.5, detailed=False))
-        self.assertEqual('0 seconds', humanfriendly.format_timespan(0))
-        self.assertEqual('0.54 seconds', humanfriendly.format_timespan(0.54321))
-        self.assertEqual('1 second', humanfriendly.format_timespan(1))
-        self.assertEqual('3.14 seconds', humanfriendly.format_timespan(math.pi))
-        self.assertEqual('1 minute', humanfriendly.format_timespan(minute))
-        self.assertEqual('1 minute and 20 seconds', humanfriendly.format_timespan(80))
-        self.assertEqual('2 minutes', humanfriendly.format_timespan(minute * 2))
-        self.assertEqual('1 hour', humanfriendly.format_timespan(hour))
-        self.assertEqual('2 hours', humanfriendly.format_timespan(hour * 2))
-        self.assertEqual('1 day', humanfriendly.format_timespan(day))
-        self.assertEqual('2 days', humanfriendly.format_timespan(day * 2))
-        self.assertEqual('1 week', humanfriendly.format_timespan(week))
-        self.assertEqual('2 weeks', humanfriendly.format_timespan(week * 2))
-        self.assertEqual('1 year', humanfriendly.format_timespan(year))
-        self.assertEqual('2 years', humanfriendly.format_timespan(year * 2))
-        self.assertEqual('6 years, 5 weeks, 4 days, 3 hours, 2 minutes and 500 milliseconds',
-                         humanfriendly.format_timespan(year*6 + week*5 + day*4 + hour*3 + minute*2 + 0.5,
-                                                       detailed=True))
-        self.assertEqual(
-            '1 year, 2 weeks and 3 days',
-            humanfriendly.format_timespan(year + week * 2 + day * 3 + hour * 12))
+        assert '1 millisecond' == humanfriendly.format_timespan(0.001, detailed=True)
+        assert '500 milliseconds' == humanfriendly.format_timespan(0.5, detailed=True)
+        assert '0.5 seconds' == humanfriendly.format_timespan(0.5, detailed=False)
+        assert '0 seconds' == humanfriendly.format_timespan(0)
+        assert '0.54 seconds' == humanfriendly.format_timespan(0.54321)
+        assert '1 second' == humanfriendly.format_timespan(1)
+        assert '3.14 seconds' == humanfriendly.format_timespan(math.pi)
+        assert '1 minute' == humanfriendly.format_timespan(minute)
+        assert '1 minute and 20 seconds' == humanfriendly.format_timespan(80)
+        assert '2 minutes' == humanfriendly.format_timespan(minute * 2)
+        assert '1 hour' == humanfriendly.format_timespan(hour)
+        assert '2 hours' == humanfriendly.format_timespan(hour * 2)
+        assert '1 day' == humanfriendly.format_timespan(day)
+        assert '2 days' == humanfriendly.format_timespan(day * 2)
+        assert '1 week' == humanfriendly.format_timespan(week)
+        assert '2 weeks' == humanfriendly.format_timespan(week * 2)
+        assert '1 year' == humanfriendly.format_timespan(year)
+        assert '2 years' == humanfriendly.format_timespan(year * 2)
+        assert '6 years, 5 weeks, 4 days, 3 hours, 2 minutes and 500 milliseconds' == \
+            humanfriendly.format_timespan(year * 6 + week * 5 + day * 4 + hour * 3 + minute * 2 + 0.5, detailed=True)
+        assert '1 year, 2 weeks and 3 days' == \
+            humanfriendly.format_timespan(year + week * 2 + day * 3 + hour * 12)
+        # Make sure milliseconds are never shown separately when detailed=False.
+        # https://github.com/xolox/python-humanfriendly/issues/10
+        assert '1 minute, 1 second and 100 milliseconds' == humanfriendly.format_timespan(61.10, detailed=True)
+        assert '1 minute and 1.1 second' == humanfriendly.format_timespan(61.10, detailed=False)
+        # Test for loss of precision as reported in issue 11:
+        # https://github.com/xolox/python-humanfriendly/issues/11
+        assert '1 minute and 0.3 seconds' == humanfriendly.format_timespan(60.300)
+        assert '5 minutes and 0.3 seconds' == humanfriendly.format_timespan(300.300)
+        assert '1 second and 15 milliseconds' == humanfriendly.format_timespan(1.015, detailed=True)
+        assert '10 seconds and 15 milliseconds' == humanfriendly.format_timespan(10.015, detailed=True)
 
     def test_parse_timespan(self):
         """Test :func:`humanfriendly.parse_timespan()`."""
@@ -153,14 +164,14 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual(0.5, humanfriendly.parse_timespan('0.5 seconds'))
         self.assertEqual(5, humanfriendly.parse_timespan('5s'))
         self.assertEqual(5, humanfriendly.parse_timespan('5 seconds'))
-        self.assertEqual(60*2, humanfriendly.parse_timespan('2m'))
-        self.assertEqual(60*2, humanfriendly.parse_timespan('2 minutes'))
-        self.assertEqual(60*60*3, humanfriendly.parse_timespan('3 h'))
-        self.assertEqual(60*60*3, humanfriendly.parse_timespan('3 hours'))
-        self.assertEqual(60*60*24*4, humanfriendly.parse_timespan('4d'))
-        self.assertEqual(60*60*24*4, humanfriendly.parse_timespan('4 days'))
-        self.assertEqual(60*60*24*7*5, humanfriendly.parse_timespan('5 w'))
-        self.assertEqual(60*60*24*7*5, humanfriendly.parse_timespan('5 weeks'))
+        self.assertEqual(60 * 2, humanfriendly.parse_timespan('2m'))
+        self.assertEqual(60 * 2, humanfriendly.parse_timespan('2 minutes'))
+        self.assertEqual(60 * 60 * 3, humanfriendly.parse_timespan('3 h'))
+        self.assertEqual(60 * 60 * 3, humanfriendly.parse_timespan('3 hours'))
+        self.assertEqual(60 * 60 * 24 * 4, humanfriendly.parse_timespan('4d'))
+        self.assertEqual(60 * 60 * 24 * 4, humanfriendly.parse_timespan('4 days'))
+        self.assertEqual(60 * 60 * 24 * 7 * 5, humanfriendly.parse_timespan('5 w'))
+        self.assertEqual(60 * 60 * 24 * 7 * 5, humanfriendly.parse_timespan('5 weeks'))
         self.assertRaises(humanfriendly.InvalidTimespan, humanfriendly.parse_timespan, '1z')
 
     def test_parse_date(self):
@@ -174,22 +185,32 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual('0 bytes', humanfriendly.format_size(0))
         self.assertEqual('1 byte', humanfriendly.format_size(1))
         self.assertEqual('42 bytes', humanfriendly.format_size(42))
-        self.assertEqual('1 KB', humanfriendly.format_size(1024 ** 1))
-        self.assertEqual('1 MB', humanfriendly.format_size(1024 ** 2))
-        self.assertEqual('1 GB', humanfriendly.format_size(1024 ** 3))
-        self.assertEqual('1 TB', humanfriendly.format_size(1024 ** 4))
-        self.assertEqual('1 PB', humanfriendly.format_size(1024 ** 5))
+        self.assertEqual('1 KB', humanfriendly.format_size(1000 ** 1))
+        self.assertEqual('1 MB', humanfriendly.format_size(1000 ** 2))
+        self.assertEqual('1 GB', humanfriendly.format_size(1000 ** 3))
+        self.assertEqual('1 TB', humanfriendly.format_size(1000 ** 4))
+        self.assertEqual('1 PB', humanfriendly.format_size(1000 ** 5))
+        self.assertEqual('1 KiB', humanfriendly.format_size(1024 ** 1, binary=True))
+        self.assertEqual('1 MiB', humanfriendly.format_size(1024 ** 2, binary=True))
+        self.assertEqual('1 GiB', humanfriendly.format_size(1024 ** 3, binary=True))
+        self.assertEqual('1 TiB', humanfriendly.format_size(1024 ** 4, binary=True))
+        self.assertEqual('1 PiB', humanfriendly.format_size(1024 ** 5, binary=True))
+        self.assertEqual('45 KB', humanfriendly.format_size(1000 * 45))
+        self.assertEqual('2.9 TB', humanfriendly.format_size(1000 ** 4 * 2.9))
 
     def test_parse_size(self):
         """Test :func:`humanfriendly.parse_size()`."""
         self.assertEqual(0, humanfriendly.parse_size('0B'))
         self.assertEqual(42, humanfriendly.parse_size('42'))
         self.assertEqual(42, humanfriendly.parse_size('42B'))
-        self.assertEqual(1024, humanfriendly.parse_size('1k'))
-        self.assertEqual(1024, humanfriendly.parse_size('1 KB'))
-        self.assertEqual(1024, humanfriendly.parse_size('1 kilobyte'))
-        self.assertEqual(1024 ** 3, humanfriendly.parse_size('1 GB'))
-        self.assertEqual(1024 ** 3 * 1.5, humanfriendly.parse_size('1.5 GB'))
+        self.assertEqual(1000, humanfriendly.parse_size('1k'))
+        self.assertEqual(1024, humanfriendly.parse_size('1k', binary=True))
+        self.assertEqual(1000, humanfriendly.parse_size('1 KB'))
+        self.assertEqual(1000, humanfriendly.parse_size('1 kilobyte'))
+        self.assertEqual(1024, humanfriendly.parse_size('1 kilobyte', binary=True))
+        self.assertEqual(1000 ** 2 * 69, humanfriendly.parse_size('69 MB'))
+        self.assertEqual(1000 ** 3, humanfriendly.parse_size('1 GB'))
+        self.assertEqual(1000 ** 3 * 1.5, humanfriendly.parse_size('1.5 GB'))
         self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, '1z')
         self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, 'a')
 
@@ -367,13 +388,13 @@ class HumanFriendlyTestCase(unittest.TestCase):
         for seconds, text in ((1, '1 second'),
                               (2, '2 seconds'),
                               (60, '1 minute'),
-                              (60*2, '2 minutes'),
-                              (60*60, '1 hour'),
-                              (60*60*2, '2 hours'),
-                              (60*60*24, '1 day'),
-                              (60*60*24*2, '2 days'),
-                              (60*60*24*7, '1 week'),
-                              (60*60*24*7*2, '2 weeks')):
+                              (60 * 2, '2 minutes'),
+                              (60 * 60, '1 hour'),
+                              (60 * 60 * 2, '2 hours'),
+                              (60 * 60 * 24, '1 day'),
+                              (60 * 60 * 24 * 2, '2 days'),
+                              (60 * 60 * 24 * 7, '1 week'),
+                              (60 * 60 * 24 * 7 * 2, '2 weeks')):
             t = humanfriendly.Timer(time.time() - seconds)
             self.assertEqual(humanfriendly.round_number(t.elapsed_time, keep_width=True), '%i.00' % seconds)
             self.assertEqual(str(t), text)
@@ -522,7 +543,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         returncode, output = main('--format-number=1234567')
         assert output.strip() == '1,234,567'
         # Test `humanfriendly --format-size'.
-        random_byte_count = random.randint(1024, 1024*1024)
+        random_byte_count = random.randint(1024, 1024 * 1024)
         returncode, output = main('--format-size=%i' % random_byte_count)
         assert output.strip() == humanfriendly.format_size(random_byte_count)
         # Test `humanfriendly --format-table'.
@@ -625,6 +646,28 @@ class HumanFriendlyTestCase(unittest.TestCase):
         """Test :func:`humanfriendly.terminal.connected_to_terminal()`."""
         self.check_terminal_capabilities(connected_to_terminal)
 
+    def test_show_pager(self):
+        """Test :func:`humanfriendly.terminal.show_pager()`."""
+        original_pager = os.environ.get('PAGER', None)
+        try:
+            # We specifically avoid `less' because it would become awkward to
+            # run the test suite in an interactive terminal :-).
+            os.environ['PAGER'] = 'cat'
+            # Generate a significant amount of random text spread over multiple
+            # lines that we expect to be reported literally on the terminal.
+            random_text = "\n".join(random_string(25) for i in range(50))
+            # Run the pager command and validate the output.
+            with CaptureOutput() as capturer:
+                show_pager(random_text)
+                assert random_text in capturer.get_text()
+        finally:
+            if original_pager is not None:
+                # Restore the original $PAGER value.
+                os.environ['PAGER'] = original_pager
+            else:
+                # Clear the custom $PAGER value.
+                os.environ.pop('PAGER')
+
     def test_terminal_supports_colors(self):
         """Test :func:`humanfriendly.terminal.terminal_supports_colors()`."""
         self.check_terminal_capabilities(terminal_supports_colors)
@@ -653,6 +696,64 @@ class HumanFriendlyTestCase(unittest.TestCase):
             This line doesn't contain a META variable.
         """)) == sorted(['VALUE', 'BYTES', 'SECONDS'])
 
+    def test_parse_usage_simple(self):
+        """Test :func:`humanfriendly.usage.parse_usage()` (a simple case)."""
+        introduction, options = self.preprocess_parse_result("""
+            Usage: my-fancy-app [OPTIONS]
+
+            Boring description.
+
+            Supported options:
+
+              -h, --help
+
+                Show this message and exit.
+        """)
+        # The following fragments are (expected to be) part of the introduction.
+        assert "Usage: my-fancy-app [OPTIONS]" in introduction
+        assert "Boring description." in introduction
+        assert "Supported options:" in introduction
+        # The following fragments are (expected to be) part of the documented options.
+        assert "-h, --help" in options
+        assert "Show this message and exit." in options
+
+    def test_parse_usage_tricky(self):
+        """Test :func:`humanfriendly.usage.parse_usage()` (a tricky case)."""
+        introduction, options = self.preprocess_parse_result("""
+            Usage: my-fancy-app [OPTIONS]
+
+            Here's the introduction to my-fancy-app. Some of the lines in the
+            introduction start with a command line option just to confuse the
+            parsing algorithm :-)
+
+            For example
+            --an-awesome-option
+            is still part of the introduction.
+
+            Supported options:
+
+              -a, --an-awesome-option
+
+                Explanation why this is an awesome option.
+
+              -b, --a-boring-option
+
+                Explanation why this is a boring option.
+        """)
+        # The following fragments are (expected to be) part of the introduction.
+        assert "Usage: my-fancy-app [OPTIONS]" in introduction
+        assert any('still part of the introduction' in p for p in introduction)
+        assert "Supported options:" in introduction
+        # The following fragments are (expected to be) part of the documented options.
+        assert "-a, --an-awesome-option" in options
+        assert "Explanation why this is an awesome option." in options
+        assert "-b, --a-boring-option" in options
+        assert "Explanation why this is a boring option." in options
+
+    def preprocess_parse_result(self, text):
+        """Ignore leading/trailing whitespace in usage parsing tests."""
+        return tuple([p.strip() for p in r] for r in parse_usage(dedent(text)))
+
     def test_format_usage(self):
         """Test :func:`humanfriendly.usage.format_usage()`."""
         # Test that options are highlighted.
@@ -689,13 +790,13 @@ class HumanFriendlyTestCase(unittest.TestCase):
               $ echo test
               test
         """))
-        assert all(token in render_usage("""
+        assert all(token in render_usage(dedent("""
             Supported options:
 
               -n, --dry-run
 
                 Don't change anything.
-        """) for token in ('`-n`', '`--dry-run`'))
+        """)) for token in ('`-n`', '`--dry-run`'))
 
     def test_sphinx_customizations(self):
         """Test the :mod:`humanfriendly.sphinx` module."""
@@ -791,6 +892,11 @@ def normalize_timestamp(value, ndigits=1):
     return '%.2f' % round(float(value), ndigits=ndigits)
 
 
+def random_string(length=25):
+    """Generate a random string."""
+    return ''.join(random.choice(string.ascii_letters) for i in range(length))
+
+
 class PatchedAttribute(object):
 
     """Context manager that temporary replaces an object attribute."""
